/*******************************************************************************
*    getMail 1.0.                                                              *
*    Copyright (C) 2002 Luca Bertoncello                                       *
*    Waltherstrae, 11D - 01067 Dresden Deutschland                            *
*    E-Mail: lucabert@lucabert.com, lucabert@lucabert.de                       *
*    http://www.lucabert.com/  http://www.lucabert.de/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define SOCKET int
#define closesocket close
#define INVALID_SOCKET  (SOCKET)(~0)
#define bzero(A,B) (memset(A, 0, B))

SOCKET connectToSMTPServer(char *host)
{
	struct sockaddr_in serv_addr;
	struct hostent *dati;
  SOCKET sock;

	bzero((char*) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family    = AF_INET;
	serv_addr.sin_port      = htons((ushort) 25);
	if((serv_addr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE )
		{
      if((dati = (struct hostent*) gethostbyname(host)) == NULL)
        {
          printf("Can't resolve '%s'\n", host);
          return(-3);
        }
      serv_addr.sin_addr.s_addr = *(long*) dati->h_addr;
		}
  if((sock = socket(AF_INET, SOCK_STREAM, 0)) != 0)
    {
      if(connect(sock, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) >= 0)
        {
          return sock;
        }
      return -2;
    }
	
	return -1;
}

int writeToSocket(SOCKET sock, char *str)
{
  return send(sock, str, strlen(str), 0);
}

char *readLineFromSocket(SOCKET sock)
{
  char line[1024], ch[2];
  int nB, nCh = 0;

  bzero(line, 1024);
  bzero(ch, 2);
  while(ch[0] != '\n')
    {
      bzero(ch, 2);
      nB = recv(sock, ch, 1, 0);
      if((nB > 0) && (ch[0] != '\n'))
        line[nCh++] = ch[0];
    }

  return strdup(line);
}

int getNumberOfMsg(SOCKET sock)
{
  char str[1024];
  int nMsg = 0;

  writeToSocket(sock, "list\n");
  readLineFromSocket(sock);
  bzero(str, 1024);
  while(str[0] != '.')
    {
      strcpy(str, readLineFromSocket(sock));
      if(str[0] != '.')
        nMsg++;
    }

  return nMsg;
}

void getAndSendMsg(SOCKET sock, int msg, char *forwardTo, char *smtpHost, char *removeFlag)
{
  char str[1024];
  SOCKET smtpSock;
  int nSent, nBytes = 1;

  printf("Forwarding msg: %d...\n", msg);
  smtpSock = connectToSMTPServer(smtpHost);
  writeToSocket(smtpSock, "helo smtp\n");
  readLineFromSocket(smtpSock);
  sprintf(str, "mail from: %s\n", forwardTo);
  writeToSocket(smtpSock, str);
  readLineFromSocket(smtpSock);
  sprintf(str, "rcpt to: %s\n", forwardTo);
  writeToSocket(smtpSock, str);
  readLineFromSocket(smtpSock);
  writeToSocket(smtpSock, "data\n");
  readLineFromSocket(smtpSock);

  sprintf(str, "retr %d\n", msg);
  writeToSocket(sock, str);
  readLineFromSocket(sock);
  bzero(str, 1024);
  while(strstr(str, "\n.\0e") == NULL)
    {
      bzero(str, 1024);
      nBytes = recv(sock, str, 1024, 0);
      nSent = 0;
      while(nSent < nBytes)
        nSent += send(smtpSock, str, nBytes, 0);
    }

  writeToSocket(smtpSock, "quit\n");
  readLineFromSocket(smtpSock);
  closesocket(smtpSock);

  if(strcmp(removeFlag, "1") == 0)
    {
      sprintf(str, "dele %d\n", msg);
      writeToSocket(sock, str);
      readLineFromSocket(sock);
    }
}

void readAndForward(SOCKET sock, char *popHost, char *remoteUser, char *remotePwd, char *forwardTo, char *smtpHost, char *removeFlag)
{
  char str[1024];
  int nMsg, msg;

  bzero(str, 1024);
  strcpy(str, readLineFromSocket(sock));
  if(strncmp(str, "+OK", 3) == 0)
    {
      sprintf(str, "user %s\n", remoteUser);
      writeToSocket(sock, str);
      bzero(str, 1024);
      strcpy(str, readLineFromSocket(sock));
      if(strncmp(str, "+OK", 3) == 0)
        {
          sprintf(str, "pass %s\n", remotePwd);
          writeToSocket(sock, str);
          bzero(str, 1024);
          strcpy(str, readLineFromSocket(sock));
          if(strncmp(str, "+OK", 3) == 0)
            {
              nMsg = getNumberOfMsg(sock);
              printf("%d message(s) on server %s\n", nMsg, popHost);
              if(nMsg != 0)
                {
                  printf("Forwarding all messages to %s, using %s\n", forwardTo, smtpHost);
                  for(msg = 1; msg <= nMsg; msg++)
                    getAndSendMsg(sock, msg, forwardTo, smtpHost, removeFlag);
                }
            }
          else
            printf("Invalid password for user %s\n", popHost);
        }
      else
        printf("Invalid user %s\n", remoteUser);
    }
  else
    printf("Unable to talk with %s\n", remoteUser);
  writeToSocket(sock, "quit\n");
}

int connectToPOPServer(char *host, char *remoteUser, char *remotePwd, char *forwardTo, char *smtpHost, char *removeFlag)
{
	struct sockaddr_in serv_addr;
	struct hostent *dati;
  SOCKET sock;

	bzero((char*) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family    = AF_INET;
	serv_addr.sin_port      = htons((ushort) 110);
	if((serv_addr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE )
		{
      if((dati = (struct hostent*) gethostbyname(host)) == NULL)
        {
          printf("Can't resolve '%s'\n", host);
          return(-3);
        }
      serv_addr.sin_addr.s_addr = *(long*) dati->h_addr;
		}
  if((sock = socket(AF_INET, SOCK_STREAM, 0)) != INVALID_SOCKET)
    {
      if(connect(sock, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) >= 0)
        {
          fcntl(sock, F_SETFL, O_NONBLOCK);
          readAndForward(sock, host, remoteUser, remotePwd, forwardTo, smtpHost, removeFlag);
          closesocket(sock);
          return 0;
        }
      return -2;
    }
	
	return -1;
}

void getMail(char *line)
{
  char *popHost;
  char *popUser;
  char *popPwd;
  char *forwardTo;
  char *smtpHost;
  char *removeFlag;

  popHost     = strtok(line, " ");
  popUser     = strtok(NULL, " ");
  popPwd      = strtok(NULL, " ");
  forwardTo   = strtok(NULL, " ");
  smtpHost    = strtok(NULL, " ");
  removeFlag  = strtok(NULL, " ");

  if(connectToPOPServer(popHost, popUser, popPwd, forwardTo, smtpHost, removeFlag) != 0)
    printf("Error contacting host\n");
}

int main(int argc, char *argv[])
{
  FILE *fp;
  char line[1024];

  if(argc < 2)
    {
      printf("Use:\n%s <configurationFile>\n", argv[0]);
      return -1;
    }
  fp = fopen(argv[1], "rt");
  if(fp != NULL)
    {
      while(!feof(fp))
        {
          fgets(line, 1024, fp);
          if(!feof(fp))
            {
              line[strlen(line) - 1] = '\0';
              if((strlen(line) != 0) && (line[0] != '#'))
                getMail(line);
            }
        }
      fclose(fp);
    }
  else
    printf("Unable to open getMail.conf\n");
}
